% Simulation Study. 
%
% created by Mingon Kang (mingon.kang@mavs.uta.edu) in 2014

clear all;

ITER = 10;
SAMPLES = 100;
SN = 5; % or 1.5

FontName = 'Helvetica';
EXPS_SEL = 1:14;                            
EXPERIMENT = length(EXPS_SEL);
PVAL = ones(ITER, EXPERIMENT, 2) * NaN;     % p-value
RPVAL = ones(ITER, EXPERIMENT, 2) * NaN;    % p-value for each r when epistasis occurs
T = cell(ITER, EXPERIMENT);                 % ground truth parameters
DATA = cell(ITER, EXPERIMENT, 2);           % simulated X and y
SNR = ones(ITER, EXPERIMENT) * NaN;         % SNR
RSQURED = ones(ITER, EXPERIMENT, 2) * NaN;  % r squared


for i = 1 : length(EXPS_SEL)

    EXPERIMENT = EXPS_SEL(i);
    % experiment setting
    switch EXPERIMENT
        case 1
            MODEL_NAME = 'MULTIPLICATIVE WB';
            R = [1 2];
        case 2
            MODEL_NAME = 'MULTIPLICATIVE WB';
            R = [1 5];
        case 3
            MODEL_NAME = 'MULTIPLICATIVE WB';
            R = [0 2];
        case 4
            MODEL_NAME = 'MULTIPLICATIVE WB';
            R = [0 5];
        case 5
            MODEL_NAME = 'MULTIPLICATIVE WB';
            R = [5 5];
        case 6
            MODEL_NAME = 'MULTIPLICATIVE WB THRESHOLD';
            R = [1 2];
        case 7
            MODEL_NAME = 'MULTIPLICATIVE WB THRESHOLD';
            R = [1 5];
        case 8
            MODEL_NAME = 'MULTIPLICATIVE WB THRESHOLD';
            R = [0 2];
        case 9
            MODEL_NAME = 'MULTIPLICATIVE WB THRESHOLD';
            R = [0 5];
        case 10
            MODEL_NAME = 'MULTIPLICATIVE WB THRESHOLD';
            R = [5 5];
        case 11
            MODEL_NAME = 'MULTIPLICATIVE';
            R = [2 2];
        case 12
            MODEL_NAME = 'MULTIPLICATIVE';
            R = [0 0];
        case 13
            MODEL_NAME = 'THRESHOLD';
            R = [2 2];
        case 14
            MODEL_NAME = 'THRESHOLD';
            R = [0 0];
    end

    iter = 1;
    while iter <= ITER
        pause(0.001);
        if mod(iter, 10) ==0
            iter 
        end

        [X, y, TB, TR, snr] = GenerateSimulationData(SAMPLES, MODEL_NAME, SN, R);
        [n p] = size(X);

        % Perform the proposed method
        % Model III and IV are performed by Model I and II by the proposed
        % model
        if strcmp(MODEL_NAME, 'MULTIPLICATIVE'), MODEL_NAME = 'MULTIPLICATIVE WB'; end
        if strcmp(MODEL_NAME, 'THRESHOLD'), MODEL_NAME = 'MULTIPLICATIVE WB THRESHOLD'; end
    
        [b r lambda stats result] = epistasis(MODEL_NAME, X, y, 1e-3, 1e+4, 1e-4, false);
        if result
            K = InterPower(MODEL_NAME, X);
            [TB(1) TB(2) TB(3) TB(4) TR(1) TR(2); b(1) b(2) b(3) b(4) r(1) r(2)]';

            % y predicted by the proposed model
            y_hat = b(1) + b(2) * X(:, 1) + b(3) * X(:, 2) + b(4) * r(1).^K(:, 1) .* r(2).^K(:, 2);
            % ground truth
            y_tr = TB(1) + TB(2) * X(:, 1) + TB(3) * X(:, 2) + TB(4) * TR(1).^K(:, 1) .* TR(2).^K(:, 2); 

            rstats = regstats(y, [X r(1) .^ K(:, 1) r(2) .^ K(:, 2)]);
        else
            continue;
        end

        % Perform with Fisher's model
        Xi = [X X(:, 1) .* X(:, 2)];
        stats_fisher = regstats(y, Xi);
        stats_fisher.tstat.pval;
        y_hat_fisher = stats_fisher.yhat;

        % Store the measurement
        SNR(iter, EXPERIMENT) = snr;
        PVAL(iter, EXPERIMENT, :) = [stats.tstat.pval(4) stats_fisher.tstat.pval(4)];
        RPVAL(iter, EXPERIMENT, :) = [rstats.tstat.pval(4) rstats.tstat.pval(5)];
        T{iter, EXPERIMENT} = [TB TR];
        DATA{iter, EXPERIMENT, 1} = X;
        DATA{iter, EXPERIMENT, 2} = y;

        RSQURED(iter, EXPERIMENT, 1) = corr(y, y_hat)^2;
        RSQURED(iter, EXPERIMENT, 2) = corr(y, y_hat_fisher)^2;
        pause(0.001);

        % reject rate
        REJECT_RATE1 = [sum(PVAL(:, :, 1) < 0.05) sum(PVAL(:, :, 2) < 0.05)];
        REJECT_RATE2 = [sum(PVAL(:, :, 1) < 0.01) sum(PVAL(:, :, 2) < 0.01)];

        iter = iter + 1;
    end

end


%% Show the result figure
LOGPVAL = -log10([PVAL(:, EXPS_SEL, 1) PVAL(:, EXPS_SEL, 2)]);
EXPS = size(LOGPVAL, 2) / 2;

hFig = figure(1);
clf(1)
set(hFig, 'Position', [1 500 600 350])

pos = zeros(EXPS * 2, 1);
sh = 0.2;
for i = 1 : EXPS * 2
    pos(i) = i;
    if mod(i, 2) ~= 0
        pos(i) = pos(i) + sh;
    else
        pos(i) = pos(i) - sh;
    end
end
data = zeros(size(LOGPVAL, 1), EXPS);
for k = 1 : EXPS
    data(:, 2*(k-1)+1:2*k) = [LOGPVAL(:, k) LOGPVAL(:, length(EXPS_SEL)+k)];
end

colors = repmat([1 0.64 0;0.67 1 0.18], EXPS, 1);

h = boxplot(data, ...
    'labelverbosity', 'minor', ...
    'outliersize', 1, 'notch', 'on', ...
    'symbol', 'w', ...
    'colors', colors, ...
    'widths', 0.5, ...
    'positions', pos);
set(h,'LineWidth', 1.5)
h = findobj(gca, 'Tag', 'Box');
for j=1:length(h)
    patch(get(h(j), 'XData'), get(h(j), 'YData'), 'y', 'FaceAlpha', 1, 'FaceColor', colors(length(h)-j+1, :));
end

tickLabelStr = {'', '1', '', '2', '', '3', '', '4', '', '5', '', '6', '', '7', ...
                '', '8', '', '9', '', '10', '', '11', '', '12', '', '13', '', '14'};

text_h = findobj(gca, 'Type', 'text');
rotation = 45;
fontSize = 12;
for cnt = 1:length(text_h)
    no = str2num(get(text_h(cnt), 'String'));
    if ~isempty(no)
        set(text_h(cnt),    'FontSize', fontSize,...
                            'FontName', FontName, ...
                            'String', tickLabelStr{no}, ...
                            'HorizontalAlignment', 'right')
    end
end
ax = axis();
axis([ax(1) ax(2) ax(3) ax(4)+10]);

squeeze = 0.2;
left = 0.02;
right = 1;
bottom = squeeze;
top = 1-squeeze;
set(gca, 'Position', [0.2 0.2 0.7 0.7])

ylabel('-log(P-value)', 'Fontname', FontName, 'FontSize', 12);
xlabel('Case Study No.', 'Fontname', FontName, 'FontSize', 12);
box = findobj(gca, 'Tag', 'Box');
h = legend(box(2:3), 'Proposed model', 'Fisher''s model', 'Fontname', FontName, ...
    'Orientation','horizontal');
set(h, 'Fontname', FontName, 'FontSize',12);


%% Draw reject rate figure
hFig = figure(2);
set(hFig, 'Position', [301 500 600 350])

REJECT_RATE = [sum(PVAL(:, EXPS_SEL, 1) < 0.01) sum(PVAL(:, EXPS_SEL, 2) < 0.01)] / ITER;
data = zeros(EXPS, 2);
for i = 1 : EXPS
    data(i, :) = [REJECT_RATE(:, i) REJECT_RATE(:, length(EXPS_SEL)+i)];
end

h = bar(data, 0.9);
set(findobj('type','axes'), 'Fontname', FontName, 'fontsize', 12)
colormap([1 0.64 0;0.67 1 0.18])
legend('Proposed model', 'Fisher''s model', 'Fontname', FontName, 'Orientation','horizontal'); %, 'Location', 'SouthEast');
ylabel('Reject Rate', 'Fontname', FontName, 'FontSize', 12);

tickLabelStr = {'1', '2', '3', '4', '5', '6', '7', ...
                '8', '9', '10', '11', '12', '13', '14'};
tickLabelStr = tickLabelStr(EXPS_SEL);
set(gca(), 'XTickLabel', tickLabelStr)

ax = axis;
axis([0.5 14.5 ax(3) 1.2]);

squeeze = 0.2;
left = 0.02;
right = 1;
bottom = squeeze;
top = 1-squeeze;
set(gca, 'Position', [0.2 0.2 0.7 0.7])

