% Simulation Study for FDR
% created by Mingon Kang (mingon.kang@mavs.uta.edu) in 2014

clear all
clc

ITER = 100;
SAMPLES = 100;
EXPERIMENT = 14;
SN = 1.5; % or 1.5

PVAL = ones(ITER, EXPERIMENT, 2) * NaN;
DATA = cell(ITER, EXPERIMENT, 2);
SNR = ones(ITER, EXPERIMENT) * NaN;
EXPS_SEL = 1:14;

for i = 1 : length(EXPS_SEL)

    EXPERIMENT = EXPS_SEL(i);
    switch EXPERIMENT
        case 1
            MODEL_NAME = 'MULTIPLICATIVE WB';
            R = [1 2];
        case 2
            MODEL_NAME = 'MULTIPLICATIVE WB';
            R = [1 5];
        case 3
            MODEL_NAME = 'MULTIPLICATIVE WB';
            R = [0 2];
        case 4
            MODEL_NAME = 'MULTIPLICATIVE WB';
            R = [0 5];
        case 5
            MODEL_NAME = 'MULTIPLICATIVE WB';
            R = [5 5];
        case 6
            MODEL_NAME = 'MULTIPLICATIVE WB THRESHOLD';
            R = [1 2];
        case 7
            MODEL_NAME = 'MULTIPLICATIVE WB THRESHOLD';
            R = [1 5];
        case 8
            MODEL_NAME = 'MULTIPLICATIVE WB THRESHOLD';
            R = [0 2];
        case 9
            MODEL_NAME = 'MULTIPLICATIVE WB THRESHOLD';
            R = [0 5];
        case 10
            MODEL_NAME = 'MULTIPLICATIVE WB THRESHOLD';
            R = [5 5];
        case 11
            MODEL_NAME = 'MULTIPLICATIVE';
            R = [2 2];
        case 12
            MODEL_NAME = 'MULTIPLICATIVE';
            R = [0 0];
        case 13
            MODEL_NAME = 'THRESHOLD';
            R = [2 2];
        case 14
            MODEL_NAME = 'THRESHOLD';
            R = [0 0];
    end

    iter = 1;
    while iter <= ITER
        pause(0.001);
        if mod(iter, 10) ==0
            iter 
        end

        % generate gene expression without interaction effects
        X = randi(3, SAMPLES, 2) - 1;
        b_a = 1;
        b_b = 1;
        b_ab = 0;
        b0 = 0;

        y = b0 + b_a .* X(:, 1) + b_b .* X(:, 2);
        STD = std(y) / SN;
        noise = STD * randn(SAMPLES, 1);
        y = y + noise;
        y(y<=0) = 0;


        [n p] = size(X);

        % Perform the proposed method
        [b r lambda stats result] = epistasis(MODEL_NAME, X, y, 5e-1, 1e+3, 1e-3, false);
        if result
            K = InterPower(MODEL_NAME, X);
            y_hat = b(1) + b(2) * X(:, 1) + b(3) * X(:, 2) + b(4) * r(1).^K(:, 1) .* r(2).^K(:, 2);
        else
            continue;
        end

        % Perform the interaction model
        Xi = X;
        statsm = regstats(y, Xi);
        y_hatm = statsm.yhat;

        Xi = [X X(:, 1) .* X(:, 2)];
        statsi = regstats(y, Xi);
        y_hati = statsi.yhat;

        % measurement
        PVAL(iter, EXPERIMENT, 1) = stats.tstat.pval(4);
        PVAL(iter, EXPERIMENT, 2) = statsi.tstat.pval(4);
        DATA{iter, EXPERIMENT, 1} = X;
        DATA{iter, EXPERIMENT, 2} = y;
        pause(0.001);

        REJECT_RATE1 = [sum(PVAL(:, :, 1) < 0.05) sum(PVAL(:, :, 2) < 0.05)];
        REJECT_RATE2 = [sum(PVAL(:, :, 1) < 0.01) sum(PVAL(:, :, 2) < 0.01)];

        iter = iter + 1;
    end

end
%%

FontName = 'Helvetica';

hFig = figure(3);
set(hFig, 'Position', [301 600 600 350])

REJECT_RATE = [sum(PVAL(:, EXPS_SEL, 1) < 0.01) sum(PVAL(:, EXPS_SEL, 2) < 0.01)] / ITER;
data = zeros(EXPERIMENT, 2);
for i = 1 : EXPERIMENT
    data(i, :) = [REJECT_RATE(:, i) REJECT_RATE(:, length(EXPS_SEL)+i)];
end

h = bar(data, 0.9);
set(findobj('type','axes'), 'Fontname', FontName, 'fontsize', 12)
colormap([1 0.64 0;0.67 1 0.18])
legend('Proposed model', 'Fisher''s model', 'Fontname', FontName, ...
    'Location', 'NorthEast', 'Orientation','horizontal');
ylabel('FDR', 'Fontname', FontName, 'FontSize', 12);
xlabel('Case Study No.', 'Fontname', FontName, 'FontSize', 12);
tickLabelStr = {'1', '2', '3', '4', '5', '6', '7', ...
                '8', '9', '10', '11', '12', '13', '14'};

tickLabelStr = tickLabelStr(EXPS_SEL);

set(gca(), 'XTickLabel', tickLabelStr)

ax = axis;
axis([0.5 14.5 ax(3) 0.1]);

squeeze = 0.2;
left = 0.02;
right = 1;
bottom = squeeze;
top = 1-squeeze;
set(gca, 'Position', [0.2 0.2 0.7 0.7])
text(-1.5, 0.1, 'D', 'FontWeight', 'bold', 'Fontname', FontName, 'FontSize', 18)
