function [X, y, B, R, SNR] = GenerateSimulationData(n, MODEL_NAME, SNR, R)
%generates simulation data based on real genome data.
%
% [X, y, B, R, SNR] = GenerateSimulationData(n, MODEL_NAME, SNR, R)
% n: sample number to generate
% MODEL_NAME: 'MULTIPLICATIVE WB' for Model I, 'MULTIPLICATIVE WB
% THRESHOLD' for Model II, MULTIPLICATIVE for Model III, and 'THRESHOLD'
% for Model IV.
% SNR: signal to noise ratio, the smaller numbers represent ther larger noise.
% R: the genetic effect size for each loci 
%
% created by Mingon Kang (mingon.kang@mavs.uta.edu) in 2014

if nargin < 3, SNR = 5; end
if nargin < 4
    r_a = randi(5, 1);
    r_b = randi(5, 1);
else
    r_a = R(1) + abs(0.1 * randn);
    r_b = R(2) + abs(0.1 * randn);
end

p = 2;
X = randi(3, n, p) - 1;

b_a = 0;
b_b = 0;
b_ab = 1;
b0 = 0;

K = InterPower(MODEL_NAME, X);
if strcmp(MODEL_NAME, 'MULTIPLICATIVE') || strcmp(MODEL_NAME, 'THRESHOLD')
    r_b = r_a;
    y = b0 + b_a .* X(:, 1) + b_b .* X(:, 2) + b_ab .* r_a.^K(:, 1);
else
    y = b0 + b_a .* X(:, 1) + b_b .* X(:, 2) + b_ab .* r_a.^K(:, 1) .* r_b.^K(:, 2);%InterEffect(MODEL_NAME, X(:, 1), X(:, 2), r_a, r_b);
end

STD = std(y) / SNR;
noise = STD * randn(n, 1);
SNR = std(y) / std(noise);
y = y + noise;
y(y<=0) = 0;

B = [b0, b_a, b_b, b_ab];
R = [r_a, r_b];