function effect = InterPower(MODEL_NAME, x)
%return the interactive effect between two loci for each model
%
% effect = Inter(MODEL_NAME, AA, BB)
%   MODEL_NAME: 
%       'MULTIPLICATIVE WB' - multiplicative within and
%                                   between loci
%       'MULTIPLICATIVE WB THRESHOLD' - multiplicative within and
%                                   between loci with threshold
%       'MULTIPLICATIVE' - two-locus interaction multiplicative effects
%       'THRESHOLD'      - two-locus interaction threshold effects
%   AA: a vector containing the number of minor1 allele
%   BB: a vector containing the number of minor2 allele
%   r:  the effect which each allele has

effect = zeros(size(x));

switch MODEL_NAME
    case 'MULTIPLICATIVE WB'
        effect = x;
    case 'MULTIPLICATIVE WB THRESHOLD'
        effect = x;
        effect(effect == 2) = 1;
    case 'MULTIPLICATIVE'
        effect = x(:, 1) .* x(:, 2);
%         effect = effect / 2;
    case 'THRESHOLD'
        effect = ones(size(x, 1), 1);
        effect(x(:, 1) == 0 | x(:, 2) == 0, 1) = 0;
    otherwise 
        error(sprintf('"%s" is not supported.', MODEL_NAME));
end

if size(x, 2) == 2 && size(effect, 2) == 1
    effect = [effect effect];
end